# Microeconometrics
# CLASS 4
# Discrete choice models in Apollo
# Wiktor Budzinski, Marek Giergiczny

rm(list=ls())
library(apollo)
library(readxl) 
library(tidyr)
library(matlib)

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 4") 
data_long <- read_excel("Fishmode.xlsx")

################################################# Exercise 1 ########################################
# 1.1 Transform long format data to short format

database <- pivot_wider(data = data_long, id_cols = id, names_from = alt, 
                        values_from = c("ans","price", "crate"))
database$choice <- database$ans_1 + 2*database$ans_2 + 3*database$ans_3 + 4*database$ans_4 # creating choice variable
database$inc <- data_long$inc[data_long$alt == 1]

# 1.2 Basic analysis of choice data

table(database$choice)/1182
barplot(table(database$choice)/1182, names.arg=c("Beach", "Pier", "Private boat", "Charter boat"))
barplot(colMeans(database[, c(6, 7, 8, 9)]), names.arg=c("Beach", "Pier", "Private boat", "Charter boat"), main="Price by mode")

Vec <- c(colMeans(database[database$choice == 1, 6]), colMeans(database[database$choice == 2, 7]), colMeans(database[database$choice == 3, 8]), colMeans(database[database$choice == 4, 9]))
barplot(Vec, names.arg=c("Beach", "Pier", "Private boat", "Charter boat"), main="Price by mode (chosen)")

barplot(colMeans(database[, c(10, 11, 12, 13)]), names.arg=c("Beach", "Pier", "Private boat", "Charter boat"), main="Crate by mode")
Vec2 <- c(colMeans(database[database$choice == 1, 10]), colMeans(database[database$choice == 2, 11]), colMeans(database[database$choice == 3, 12]), colMeans(database[database$choice == 4, 13]))
barplot(Vec2, names.arg=c("Beach", "Pier", "Private boat", "Charter boat"), main="Crate by mode (chosen)")

# 1.3 Estimation of MNL

# Output settings
modelOutput_settings=list()
modelOutput_settings[['printPVal']] <- 2 # Print p-values
modelOutput_settings[['printClassical']] <- FALSE

apollo_initialise()
apollo_control = list(modelName = "MNL1", modelDescr = "Fishmode MNL", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

mnl1 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnl1, modelOutput_settings)
apollo_saveOutput(mnl1, modelOutput_settings) # Important to save that for starting values

# 1.4 Calculate WTP

WTP_crate<-apollo_deltaMethod(mnl1,deltaMethod_settings = list(operation="ratio",parName1="bcrate", parName2="bprice",multPar1=-1))
WTP_charter<-apollo_deltaMethod(mnl1,deltaMethod_settings = list(operation="ratio",parName1="asc_charter", parName2="bprice",multPar1=-1))

# 1.5 Calculate marginal effects

### Use the estimated model to make predictions
predictions_base = apollo_prediction(mnl1, apollo_probabilities, apollo_inputs)

### The cost of chartered boat increases by 1%
database$price_4 = 1.01*database$price_4

### Rerun predictions with the new data
apollo_inputs = apollo_validateInputs()
predictions_new = apollo_prediction(mnl1, apollo_probabilities, apollo_inputs)

### Return to original data
database$price_4 = (1/1.01)*database$price_4
apollo_inputs = apollo_validateInputs()

### Compute change in probabilities (elasticity)
change <- (predictions_new-predictions_base)/predictions_base
change <- change/0.01
### Get rid of not needed columns
change=change[,-ncol(change)]
change=change[,-c(1,2)]
### Look at mean elasticity
colMeans(change)

################################################# Exercise 2 ########################################
# 2.1 Add income into the model 

apollo_control = list(modelName = "MNL2", modelDescr = "Fishmode MNL with income", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1,
              asc_pier_inc = 0, asc_private_inc = 0, asc_charter_inc = 0)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + asc_pier_inc*inc + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private  + asc_private_inc*inc + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter  + asc_charter_inc*inc + bcrate*crate_4 + bprice*price_4
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

mnl2 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnl2, modelOutput_settings)

apollo_lrTest(mnl1, mnl2) # LR test

# Interactions with attributes
database$inc <- database$inc/1000
apollo_control = list(modelName = "MNL2b", modelDescr = "Fishmode MNL with income and interactions", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1,
              asc_pier_inc = 0, asc_private_inc = 0, asc_charter_inc = 0, bcrate_inc = 0, bprice_inc = 0)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1 + bcrate_inc*crate_1*inc + bprice_inc*price_1*inc
  V[["pier"]]  = asc_pier + asc_pier_inc*inc + bcrate*crate_2 + bprice*price_2 + bcrate_inc*crate_2*inc + bprice_inc*price_2*inc
  V[["private"]]  = asc_private  + asc_private_inc*inc + bcrate*crate_3 + bprice*price_3 + bcrate_inc*crate_3*inc + bprice_inc*price_3*inc
  V[["charter"]] = asc_charter  + asc_charter_inc*inc + bcrate*crate_4 + bprice*price_4 + bcrate_inc*crate_4*inc + bprice_inc*price_4*inc
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

mnl2b = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnl2b, modelOutput_settings)

apollo_lrTest(mnl1, mnl2b) # LR test


# 2.2 Calculate WTP for different quartiles of income
summary(database$inc)
deltaMethod_settings=list(expression=c(WTP_crate_25="-(bcrate+bcrate_inc*2.08)/(bprice+bprice_inc*2.08)",
                                       WTP_crate_50="-(bcrate+bcrate_inc*3.75)/(bprice+bprice_inc*3.75)",
                                       WTP_crate_75="-(bcrate+bcrate_inc*5.42)/(bprice+bprice_inc*5.42)"))
apollo_deltaMethod(mnl2b, deltaMethod_settings)


# 2.3 How to correctly incorporate income into choice model

apollo_control = list(modelName = "MNL3", modelDescr = "Fishmode MNL with correct income", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1)
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values
apollo_beta[5]= - apollo_beta[5]
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*log(1000*inc - price_1)
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*log(1000*inc - price_2)
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*log(1000*inc - price_3)
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*log(1000*inc - price_4)
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

mnl3 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnl3, modelOutput_settings)

mnl1$maximum
mnl3$maximum

# 2.4 Add heteroskedasticity into the model  

apollo_control = list(modelName = "MNL4", modelDescr = "Fishmode Scaled MNL", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1, bscale_inc = 0.1)
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  Scale = exp(inc*bscale_inc)
  V[["beach"]]  =  Scale*(bcrate*crate_1 + bprice*price_1)
  V[["pier"]]  = Scale*(asc_pier + bcrate*crate_2 + bprice*price_2)
  V[["private"]]  = Scale*(asc_private + bcrate*crate_3 + bprice*price_3)
  V[["charter"]] = Scale*(asc_charter + bcrate*crate_4 + bprice*price_4)
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

mnl4 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnl4, modelOutput_settings)

apollo_lrTest(mnl1, mnl4) # LR test

################################################# Exercise 3 ########################################
# Test IIA assumption 

apollo_control = list(modelName = "MNL2", modelDescr = "Fishmode MNL without private boat", indivID = "id",  outputDirectory = "output")
data <- database
database <- database[database$choice != 3,]
apollo_beta=c(asc_pier = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1)
apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = c(beach=1, pier=2, charter=4), 
    avail         = list(beach=1, pier=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

mnl2 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnl2, modelOutput_settings)

CoefDiff <- mnl2$estimate - mnl1$estimate[-2] # Second model first
MatDiff <- inv(-mnl2$hessian) - inv(-mnl1$hessian[-2,-2])
H <- t(CoefDiff)  %*%  solve(MatDiff, CoefDiff)
1-pchisq(H, 4) # p-value

database <- data # restore the database

################################################# Exercise 4 ########################################
# Estimate a nested multinomial logit 
# 4.1 Fishing from land vs. fishing from boat


apollo_control = list(modelName = "NL1", modelDescr = "Fishmode Nested logit, land vs. boat", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1, lambda_land = 0.1, lambda_boat = 0.1)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Specify nests for NL model
  nlNests      = list(root=1, land= lambda_land, boat = lambda_boat)
  
  ### Specify tree structure for NL model
  nlStructure= list()
  nlStructure[["root"]]   = c("land","boat")
  nlStructure[["land"]]     = c("beach","pier")
  nlStructure[["boat"]]     = c("private","charter")
  
  ### Define settings for NL model
  nl_settings <- list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V,
    nlNests      = nlNests,
    nlStructure  = nlStructure
  )
  
  ### Compute probabilities using NL model
  P[["model"]] = apollo_nl(nl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

nl1 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(nl1, modelOutput_settings)

mnl1$maximum
nl1$maximum

# 4.1b Land vs. boats separately

apollo_control = list(modelName = "NL1b", modelDescr = "Fishmode Nested logit, land vs. boat", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1, lambda_land = 0.1)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Specify nests for NL model
  nlNests      = list(root=1, land= lambda_land)
  
  ### Specify tree structure for NL model
  nlStructure= list()
  nlStructure[["root"]]   = c("land", "private", "charter")
  nlStructure[["land"]]     = c("beach","pier")

  ### Define settings for NL model
  nl_settings <- list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V,
    nlNests      = nlNests,
    nlStructure  = nlStructure
  )
  
  ### Compute probabilities using NL model
  P[["model"]] = apollo_nl(nl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

nl1b = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(nl1b, modelOutput_settings)
mnl1$maximum
nl1b$maximum

# 4.2 Fishing with private resources vs. fishing with chartered boat

apollo_control = list(modelName = "NL2", modelDescr = "Fishmode Nested logit, private vs. chartered", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1, lambda = 0.1)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Specify nests for NL model
  nlNests      = list(root=1, priv = lambda)
  
  ### Specify tree structure for NL model
  nlStructure= list()
  nlStructure[["root"]]   = c("priv","charter")
  nlStructure[["priv"]]     = c("beach","pier", "private")

  ### Define settings for NL model
  nl_settings <- list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V,
    nlNests      = nlNests,
    nlStructure  = nlStructure
  )
  
  ### Compute probabilities using NL model
  P[["model"]] = apollo_nl(nl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

nl2 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(nl2, modelOutput_settings)

apollo_lrTest(mnl1, nl2) # LR test